#include "widget.h"

Widget::Widget()
{
    currfilepath = "";
    setupTextEdit();
}

Widget::Widget(const QString &filepath)
{
    currfilepath = filepath;
    fromFileToWidget();
    setupTextEdit();
}

void Widget::fromFileToWidget()
{
    QFile file(currfilepath);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return;
    QByteArray content = file.readAll();
    file.close();
    document()->setPlainText(content);

    document()->setModified(0);
}

void Widget::fromWidgetToFile()
{
    QFile file(currfilepath);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) return;
    QString content = document()->toPlainText();
    file.write(content.toUtf8());
    file.close();
    document()->setModified(0);
}

void Widget::fromWidgetToFile(const QString &filepath)
{
    if(!isBinded()) { highlighter = new Highlighter(document()); }
    QString oldPath = currfilepath;
    currfilepath = filepath;

    if(!highlighter->setSyntaxSettings(QFileInfo(currfilepath).suffix())) {
        QMessageBox::warning(this, tr("Ошибка"), tr("Проблема с XML файлом настроек."));
    }
    fromWidgetToFile();
    emit currentPathChanged(oldPath, currfilepath);
}

bool Widget::isBinded()
{
    return !currfilepath.isEmpty();
}

bool Widget::isModified()
{
    return document()->isModified();
}

QString Widget::getCurrentFilePath()
{
    return currfilepath;
}

void Widget::lineNumberAreaPaintEvent(QPaintEvent *event)
{
    QPainter painter(lineNumberArea);
    painter.fillRect(event->rect(), Qt::lightGray);
    QTextBlock block = firstVisibleBlock();
    int blockNumber = block.blockNumber();
    int top = qRound(blockBoundingGeometry(block).translated(contentOffset()).top());
    int bottom = top + qRound(blockBoundingRect(block).height());
    while (block.isValid() && top <= event->rect().bottom()) {
        if (block.isVisible() && bottom >= event->rect().top()) {
            QString number = QString::number(blockNumber + 1);
            painter.setPen(Qt::black);
            painter.drawText(0, top, lineNumberArea->width(), fontMetrics().height(),
                             Qt::AlignRight, number);
        }

        block = block.next();
        top = bottom;
        bottom = top + qRound(blockBoundingRect(block).height());
        ++blockNumber;
    }
}

int Widget::lineNumberAreaWidth()
{
    int digits = 1;
    int max = qMax(1, blockCount());
    while (max >= 10) {
        max /= 10;
        ++digits;
    }

    int space = 3 + fontMetrics().horizontalAdvance(QLatin1Char('9')) * digits;

    return space;
}

void Widget::updateLineNumberAreaWidth(int newBlockCount)
{
    Q_UNUSED(newBlockCount);
    setViewportMargins(lineNumberAreaWidth(), 0, 0, 0);
}

void Widget::highlightCurrentLine()
{
    QList<QTextEdit::ExtraSelection> extraSelections;

    if (!isReadOnly()) {
        QTextEdit::ExtraSelection selection;

        QColor lineColor = QColor(Qt::yellow).lighter(160);

        selection.format.setBackground(lineColor);
        selection.format.setProperty(QTextFormat::FullWidthSelection, true);
        selection.cursor = textCursor();
        selection.cursor.clearSelection();
        extraSelections.append(selection);
    }

    setExtraSelections(extraSelections);
}

void Widget::updateLineNumberArea(const QRect &rect, int dy)
{
    if (dy)
        lineNumberArea->scroll(0, dy);
    else
        lineNumberArea->update(0, rect.y(), lineNumberArea->width(), rect.height());

    if (rect.contains(viewport()->rect()))
        updateLineNumberAreaWidth(0);
}

void Widget::setupTextEdit()
{
    QFont font;
    font.setFamily("Courier");
    font.setFixedPitch(true);
    font.setPointSize(15);
    setFont(font);

    if(isBinded()) {
        highlighter = new Highlighter(document());
        if(!highlighter->setSyntaxSettings(QFileInfo(currfilepath).suffix())) {
            QMessageBox::warning(this, tr("Ошибка"), tr("Проблема с XML файлом настроек."));
        }
    }

    lineNumberArea = new LineNumberArea(this);

    connect(this, &Widget::blockCountChanged, this, &Widget::updateLineNumberAreaWidth);
    connect(this, &Widget::updateRequest, this, &Widget::updateLineNumberArea);
    connect(this, &Widget::cursorPositionChanged, this, &Widget::highlightCurrentLine);

    updateLineNumberAreaWidth(0);
    highlightCurrentLine();
}

void Widget::resizeEvent(QResizeEvent *event)
{
    QPlainTextEdit::resizeEvent(event);

    QRect cr = contentsRect();
    lineNumberArea->setGeometry(QRect(cr.left(), cr.top(), lineNumberAreaWidth(), cr.height()));
}



LineNumberArea::LineNumberArea(Widget *editor) : QWidget(editor), filewidget(editor)
{}

QSize LineNumberArea::sizeHint() const
{
    return QSize(filewidget->lineNumberAreaWidth(), 0);
}

void LineNumberArea::paintEvent(QPaintEvent *event)
{
    filewidget->lineNumberAreaPaintEvent(event);
}



